@@ -207,11 +207,13 @@ class _CachedFunction(object):
         elif isinstance(key, basestring):
             data = key
         elif hasattr(key, "__dict__"):
-            data = self._pickler(key.__dict__)
+            data = self._compute_hash(key.__dict__)
         elif isinstance(key, (bool, int, long, float,)):
-            data = self._pickler(key)
+            data = str(self._pickler(key))
         else:
             raise Exception("can not compute hash for %r" % (key,))
+        if not isinstance(data, basestring):
+            raise RuntimeError("implementation error: data should be str, but is %s" % type(data))
         muncher = hashlib.sha1()
         muncher.update(data)
         return muncher.hexdigest()
